// Use of this source code is governed by a BSD-style license
// that can be found in the License file.
//
// Author: Shuo Chen (chenshuo at chenshuo dot com)

#ifndef MUDUO_BASE_CURRENTTHREAD_H
#define MUDUO_BASE_CURRENTTHREAD_H

#include "muduo/base/Types.h"

namespace muduo {
namespace CurrentThread {
// internal
extern __thread int         t_cachedTid;
extern __thread char        t_tidString[32];
extern __thread int         t_tidStringLength;
extern __thread const char* t_threadName;

// cacheTid 用于获取当前线程的 ID 并存储到 t_cachedTid
void cacheTid();

inline int tid()
{
    // __builtin_expect 这个应该是一个优化的语句
    if (__builtin_expect(t_cachedTid == 0, 0)) {
        cacheTid();
    }
    return t_cachedTid;
}

inline const char* tidString()  // for logging
{
    return t_tidString;
}

inline int tidStringLength()  // for logging
{
    return t_tidStringLength;
}

inline const char* name() { return t_threadName; }

bool isMainThread();

void sleepUsec(int64_t usec);  // for testing

string stackTrace(bool demangle);
}  // namespace CurrentThread
}  // namespace muduo

#endif  // MUDUO_BASE_CURRENTTHREAD_H
