// Copyright 2010, Shuo Chen.  All rights reserved.
// http://code.google.com/p/muduo/
//
// Use of this source code is governed by a BSD-style license
// that can be found in the License file.

// Author: Shuo Chen (chenshuo at chenshuo dot com)
//
// This is an internal header file, you should not include this.

#ifndef MUDUO_NET_ACCEPTOR_H
#define MUDUO_NET_ACCEPTOR_H

#include <functional>

#include "muduo/net/Channel.h"
#include "muduo/net/Socket.h"

namespace muduo {
namespace net {

class EventLoop;
class InetAddress;


/*
    Acceptor class，用于 accept(2) 新 TCP 连接，并通过回调通知使用者。
    它是内部 class，供 TcpServer 使用，生命期由后者控制。
*/
///
/// Acceptor of incoming TCP connections.
///
class Acceptor : noncopyable {
public:
    typedef std::function<void(int sockfd, const InetAddress&)>
        NewConnectionCallback;

    Acceptor(EventLoop* loop, const InetAddress& listenAddr, bool reuseport);
    ~Acceptor();

    void setNewConnectionCallback(const NewConnectionCallback& cb)
    {
        newConnectionCallback_ = cb;
    }

    bool listenning() const { return listenning_; }
    void listen();

private:
    void handleRead();

/*
    Socket 是一个 RAII handle，
    封装了 socket 文件描述符的生命期。Acceptor 的 socket 是 listening socket，即
    server socket。Channel 用于观察此 socket 上的 readable 事件，并回调 Acceptor::
    handleRead()，后者会调用 accept(2) 来接受新连接，并回调用户 callback。
*/
    EventLoop*            loop_;
    Socket                acceptSocket_;
    Channel               acceptChannel_;
    NewConnectionCallback newConnectionCallback_;
    bool                  listenning_;
    int                   idleFd_;
};

}  // namespace net
}  // namespace muduo

#endif  // MUDUO_NET_ACCEPTOR_H
