// Copyright 2010, Shuo Chen.  All rights reserved.
// http://code.google.com/p/muduo/
//
// Use of this source code is governed by a BSD-style license
// that can be found in the License file.

// Author: Shuo Chen (chenshuo at chenshuo dot com)
//
// This is a public header file, it must only include public header files.

#ifndef MUDUO_NET_EVENTLOOPTHREAD_H
#define MUDUO_NET_EVENTLOOPTHREAD_H

#include "muduo/base/Condition.h"
#include "muduo/base/Mutex.h"
#include "muduo/base/Thread.h"

namespace muduo {
namespace net {

class EventLoop;

/*
    IO 线程不一定是主线程，我们可以在任何一个线程创建并运行 EventLoop。一个
    程序也可以有不止一个 IO 线程，我们可以按优先级将不同的 socket 分给不同的 IO
    线程，避免优先级反转。为了方便将来使用，我们定义 EventLoopThread class，这正
    是 one loop per thread 的本意。
*/
class EventLoopThread : noncopyable {
public:
    typedef std::function<void(EventLoop*)> ThreadInitCallback;

    EventLoopThread(const ThreadInitCallback& cb = ThreadInitCallback(),
                    const string&             name = string());
    ~EventLoopThread();
    EventLoop* startLoop();

private:
    void threadFunc();

    EventLoop* loop_   GUARDED_BY(mutex_);
    bool               exiting_;
    Thread             thread_;
    MutexLock          mutex_;
    Condition cond_    GUARDED_BY(mutex_);
    ThreadInitCallback callback_;
};

}  // namespace net
}  // namespace muduo

#endif  // MUDUO_NET_EVENTLOOPTHREAD_H
